from .fa_mteb_classification import (
    DeepSentiPers,
    DeepSentiPersV2,
    DigikalamagClassification,
    FaIntentClassification,
    NLPTwitterAnalysisClassification,
    NLPTwitterAnalysisClassificationV2,
    PerShopDomainClassification,
    PerShopIntentClassification,
    PersianTextEmotion,
    PersianTextEmotionV2,
    SentimentDKSF,
    SentimentDKSFV2,
    SIDClassification,
    SIDClassificationV2,
    StyleClassification,
    SynPerChatbotConvSAAnger,
    SynPerChatbotConvSAFear,
    SynPerChatbotConvSAFriendship,
    SynPerChatbotConvSAHappiness,
    SynPerChatbotConvSAJealousy,
    SynPerChatbotConvSALove,
    SynPerChatbotConvSASadness,
    SynPerChatbotConvSASatisfaction,
    SynPerChatbotConvSASurprise,
    SynPerChatbotConvSAToneChatbotClassification,
    SynPerChatbotConvSAToneUserClassification,
    SynPerChatbotRAGToneChatbotClassification,
    SynPerChatbotRAGToneUserClassification,
    SynPerChatbotSatisfactionLevelClassification,
    SynPerChatbotToneChatbotClassification,
    SynPerChatbotToneUserClassification,
    SynPerTextToneClassification,
    SynPerTextToneClassificationV2,
    SynPerTextToneClassificationV3,
)
from .persian_food_sentiment_classification import PersianFoodSentimentClassification

__all__ = [
    "DeepSentiPers",
    "DeepSentiPersV2",
    "DigikalamagClassification",
    "FaIntentClassification",
    "NLPTwitterAnalysisClassification",
    "NLPTwitterAnalysisClassificationV2",
    "PerShopDomainClassification",
    "PerShopIntentClassification",
    "PersianFoodSentimentClassification",
    "PersianTextEmotion",
    "PersianTextEmotionV2",
    "SIDClassification",
    "SIDClassificationV2",
    "SentimentDKSF",
    "SentimentDKSFV2",
    "StyleClassification",
    "SynPerChatbotConvSAAnger",
    "SynPerChatbotConvSAFear",
    "SynPerChatbotConvSAFriendship",
    "SynPerChatbotConvSAHappiness",
    "SynPerChatbotConvSAJealousy",
    "SynPerChatbotConvSALove",
    "SynPerChatbotConvSASadness",
    "SynPerChatbotConvSASatisfaction",
    "SynPerChatbotConvSASurprise",
    "SynPerChatbotConvSAToneChatbotClassification",
    "SynPerChatbotConvSAToneUserClassification",
    "SynPerChatbotRAGToneChatbotClassification",
    "SynPerChatbotRAGToneUserClassification",
    "SynPerChatbotSatisfactionLevelClassification",
    "SynPerChatbotToneChatbotClassification",
    "SynPerChatbotToneUserClassification",
    "SynPerTextToneClassification",
    "SynPerTextToneClassificationV2",
    "SynPerTextToneClassificationV3",
]
