from mteb.models import ModelMeta

human = ModelMeta(
    loader=None,
    name="Human",
    languages=["eng-Latn", "ara-Arab", "rus-Cyrl", "dan-Latn", "nob-Latn"],
    open_weights=True,
    revision="2025_09_25",
    release_date=None,
    n_parameters=None,
    memory_usage_mb=None,
    embed_dim=None,
    license=None,
    max_tokens=None,
    reference=None,
    similarity_fn_name=None,
    framework=[],
    use_instructions=None,
    training_datasets=None,
    public_training_code=None,
    public_training_data=None,
)
