from .argu_ana_vn_retrieval import ArguAnaVN
from .climate_fevervn_retrieval import ClimateFEVERVN
from .cqa_dupstack_android_vn_retrieval import CQADupstackAndroidVN
from .cqa_dupstack_gis_vn_retrieval import CQADupstackGisVN
from .cqa_dupstack_mathematica_vn_retrieval import CQADupstackMathematicaVN
from .cqa_dupstack_physics_vn_retrieval import CQADupstackPhysicsVN
from .cqa_dupstack_programmers_vn_retrieval import CQADupstackProgrammersRetrievalVN
from .cqa_dupstack_stats_vn_retrieval import CQADupstackStatsVN
from .cqa_dupstack_tex_vn_retrieval import CQADupstackTexVN
from .cqa_dupstack_unix_vn_retrieval import CQADupstackUnixVN
from .cqa_dupstack_webmasters_vn_retrieval import CQADupstackWebmastersVN
from .cqa_dupstack_wordpress_vn_retrieval import CQADupstackWordpressVN
from .db_pedia_vn_retrieval import DBPediaVN
from .fevervn_retrieval import FEVERVN
from .fi_qa2018_vn_retrieval import FiQA2018VN
from .green_node_table_markdown_retrieval import GreenNodeTableMarkdownRetrieval
from .hotpot_qavn_retrieval import HotpotQAVN
from .msmarcovn_retrieval import MSMARCOVN
from .nf_corpus_vn_retrieval import NFCorpusVN
from .nqvn_retrieval import NQVN
from .quora_vn_retrieval import QuoraVN
from .sci_fact_vn_retrieval import SciFactVN
from .scidocsvn_retrieval import SCIDOCSVN
from .touche2020_vn_retrieval import Touche2020VN
from .treccovidvn_retrieval import TRECCOVIDVN
from .vie_qu_ad_retrieval import VieQuADRetrieval
from .zac_legal_text_retrieval import ZacLegalTextRetrieval

__all__ = [
    "FEVERVN",
    "MSMARCOVN",
    "NQVN",
    "SCIDOCSVN",
    "TRECCOVIDVN",
    "ArguAnaVN",
    "CQADupstackAndroidVN",
    "CQADupstackGisVN",
    "CQADupstackMathematicaVN",
    "CQADupstackPhysicsVN",
    "CQADupstackProgrammersRetrievalVN",
    "CQADupstackStatsVN",
    "CQADupstackTexVN",
    "CQADupstackUnixVN",
    "CQADupstackWebmastersVN",
    "CQADupstackWordpressVN",
    "ClimateFEVERVN",
    "DBPediaVN",
    "FiQA2018VN",
    "GreenNodeTableMarkdownRetrieval",
    "HotpotQAVN",
    "NFCorpusVN",
    "QuoraVN",
    "SciFactVN",
    "Touche2020VN",
    "VieQuADRetrieval",
    "ZacLegalTextRetrieval",
]
